@@ -1,35 +1,59 @@
 #' Update a historical table
-#' @template .data_dbi
+#'
+#' @description
+#'   `update_snapshots` makes it easy to create and update a historical data table on a remote (SQL) server.
+#'   The function takes the data (`.data`) as it looks on a given point in time (`timestamp`) and then updates
+#'   (or creates) an remote table identified by `db_table`.
+#'   This update only stores the changes between the new data (`.data`) and the data currently stored on the remote.
+#'   This way, the data can be reconstructed as it looked at any point in time while taking as little space as possible.
+#'
+#'   See `vignette("basic-principles")` for further introduction to the function.
+#'
+#' @template .data
 #' @template conn
-#' @param db_table An object that inherits from `tbl_dbi`, a [DBI::Id()] object or a character string readable by [id].
-#' @param timestamp
-#'   A timestamp (POSIXct) with which to update from_ts/until_ts columns
+#' @template db_table
+#' @param timestamp (`POSIXct(1)`, `Date(1)`, or `character(1)`)\cr
+#'   The timestamp describing the data being processed (not the current time).
 #' @template filters
-#' @param message
-#'   A message to add to the log-file (useful for supplying metadata to the log)
-#' @param tic
-#'   A timestamp when computation began. If not supplied, it will be created at call-time.
-#'   (Used to more accurately convey how long runtime of the update process has been)
-#' @param logger
-#'   A [Logger] instance. If none is given, one is initialized with default arguments.
-#' @param enforce_chronological_order
-#'   A logical that controls whether or not to check if timestamp of update is prior to timestamps in the DB
-#' @return No return value, called for side effects
-#' @examples
-#' conn <- get_connection(drv = RSQLite::SQLite())
+#' @param message (`character(1)`)\cr
+#'   A message to add to the log-file (useful for supplying metadata to the log).
+#' @param tic (`POSIXct(1)`)\cr
+#'   A timestamp when computation began. If not supplied, it will be created at call-time
+#'   (used to more accurately convey the runtime of the update process).
+#' @param logger (`Logger(1)`)\cr
+#'   A configured logging object. If none is given, one is initialized with default arguments.
+#' @param enforce_chronological_order (`logical(1)`)\cr
+#'   Are updates allowed if they are chronologically earlier than latest update?
+#' @return
+#'   No return value, called for side effects.
+#' @examplesIf requireNamespace("RSQLite", quietly = TRUE)
+#'   conn <- get_connection()
 #'
-#' data <- dplyr::copy_to(conn, mtcars)
+#'   data <- dplyr::copy_to(conn, mtcars)
 #'
-#' update_snapshot(data,
-#'                 conn = conn,
-#'                 db_table = "test.mtcars",
-#'                 timestamp = Sys.time())
+#'   # Copy the first 3 records
+#'   update_snapshot(
+#'     head(data, 3),
+#'     conn = conn,
+#'     db_table = "test.mtcars",
+#'     timestamp = Sys.time()
+#'   )
 #'
-#' close_connection(conn)
+#'   # Update with the first 5 records
+#'   update_snapshot(
+#'     head(data, 5),
+#'     conn = conn,
+#'     db_table = "test.mtcars",
+#'     timestamp = Sys.time()
+#'   )
+#'
+#'   dplyr::tbl(conn, "test.mtcars")
+#'
+#'   close_connection(conn)
 #' @seealso filter_keys
 #' @importFrom rlang .data
 #' @export
-update_snapshot <- function(.data, conn, db_table, timestamp, filters = NULL, message = NULL, tic = Sys.time(), # nolint: cyclocomp_linter, line_length_linter
+update_snapshot <- function(.data, conn, db_table, timestamp, filters = NULL, message = NULL, tic = Sys.time(),
                             logger = NULL,
                             enforce_chronological_order = TRUE) {
 
@@ -41,18 +65,19 @@ update_snapshot <- function(.data, conn, db_table, timestamp, filters = NULL, me
   checkmate::assert_class(filters, "tbl_dbi", null.ok = TRUE)
   checkmate::assert_character(message, null.ok = TRUE)
   assert_timestamp_like(tic)
-  checkmate::assert_class(logger, "Logger", null.ok = TRUE)
+  checkmate::assert_multi_class(logger, "Logger", null.ok = TRUE)
   checkmate::assert_logical(enforce_chronological_order)
 
   # Retrieve Id from any valid db_table inputs to correctly create a missing table
   db_table_id <- id(db_table, conn)
-  db_table_name <- db_table_id |>
-    methods::slot("name") |>
-    stats::na.omit() |>
-    paste(collapse = ".")
 
   if (table_exists(conn, db_table_id)) {
-    db_table <- dplyr::tbl(conn, db_table_id, check_from = FALSE)
+    # Obtain a lock on the table
+    if (!lock_table(conn, db_table_id, schema = get_schema(db_table_id))) {
+      stop("A lock could not be obtained on the table")
+    }
+
+    db_table <- dplyr::tbl(conn, db_table_id)
   } else {
     db_table <- create_table(dplyr::collect(utils::head(.data, 0)), conn, db_table_id, temporary = FALSE)
   }
@@ -60,14 +85,13 @@ update_snapshot <- function(.data, conn, db_table, timestamp, filters = NULL, me
   # Initialize logger
   if (is.null(logger)) {
     logger <- Logger$new(
-      db_tablestring = db_table_name,
+      db_table = db_table_id,
       log_conn = conn,
-      ts = timestamp,
+      timestamp = timestamp,
       start_time = tic
     )
   }
 
-  logger$log_to_db(start_time = !!db_timestamp(tic, conn))
   logger$log_info("Started", tic = tic) # Use input time in log
 
   # Add message to log (if given)
@@ -79,24 +103,32 @@ update_snapshot <- function(.data, conn, db_table, timestamp, filters = NULL, me
 
   # Opening checks
   if (!is.historical(db_table)) {
+
+    # Release table lock
+    unlock_table(conn, db_table_id, get_schema(db_table_id))
+
     logger$log_to_db(success = FALSE, end_time = !!db_timestamp(tic, conn))
     logger$log_error("Table does not seem like a historical table", tic = tic) # Use input time in log
   }
 
   if (!setequal(colnames(.data),
                 colnames(dplyr::select(db_table, !c("checksum", "from_ts", "until_ts"))))) {
+
+    # Release table lock
+    unlock_table(conn, db_table_id, get_schema(db_table_id))
+
     logger$log_to_db(success = FALSE, end_time = !!db_timestamp(tic, conn))
-    logger$log_error("Columns do not match!\n",
-                     "Table columns:\n",
-                     paste(colnames(dplyr::select(db_table, !tidyselect::any_of(c("checksum", "from_ts", "until_ts")))),
-                           collapse = ", "),
-                     "\nInput columns:\n",
-                     paste(colnames(.data), collapse = ", "), tic = tic) # Use input time in log
+    logger$log_error(
+      "Columns do not match!\n",
+      "Table columns:\n",
+      toString(colnames(dplyr::select(db_table, !tidyselect::any_of(c("checksum", "from_ts", "until_ts"))))),
+      "\nInput columns:\n",
+      toString(colnames(.data)),
+      tic = tic # Use input time in log
+    )
   }
 
-  logger$log_to_db(schema = purrr::pluck(db_table_id@name, "schema"), table = purrr::pluck(db_table_id@name, "table"))
-  logger$log_info("Parsing data for table", db_table_name, "started", tic = tic) # Use input time in log
-  logger$log_to_db(date = !!db_timestamp(timestamp, conn))
+  logger$log_info("Parsing data for table", as.character(db_table_id), "started", tic = tic) # Use input time in log
   logger$log_info("Given timestamp for table is", timestamp, tic = tic) # Use input time in log
 
   # Check for current update status
@@ -111,6 +143,10 @@ update_snapshot <- function(.data, conn, db_table, timestamp, filters = NULL, me
   db_latest <- strftime(db_latest)
 
   if (enforce_chronological_order && timestamp < db_latest) {
+
+    # Release the table lock
+    unlock_table(conn, db_table_id, get_schema(db_table_id))
+
     logger$log_to_db(success = FALSE, end_time = !!db_timestamp(tic, conn))
     logger$log_error("Given timestamp", timestamp, "is earlier than latest",
                      "timestamp in table:", db_latest, tic = tic) # Use input time in log
@@ -119,15 +155,24 @@ update_snapshot <- function(.data, conn, db_table, timestamp, filters = NULL, me
   # Compute .data immediately to reduce runtime and compute checksum
   .data <- .data |>
     dplyr::ungroup() |>
-    dplyr::select(
-      colnames(dplyr::select(db_table, !tidyselect::any_of(c("checksum", "from_ts", "until_ts"))))
-    ) |>
-    digest_to_checksum(col = "checksum", warn = TRUE) |>
     filter_keys(filters) |>
-    dplyr::compute()
+    dplyr::select(colnames(dplyr::select(db_table, !tidyselect::any_of(c("checksum", "from_ts", "until_ts")))))
 
+  # Copy to the target connection if needed
+  if (!identical(dbplyr::remote_con(.data), conn)) {
+    .data <- dplyr::copy_to(conn, .data, name = unique_table_name())
+    defer_db_cleanup(.data)
+  }
+
+  # Once we ensure .data is on the same connection as the target, we compute the checksums
+  .data <- dplyr::compute(digest_to_checksum(.data, col = "checksum"))
+  defer_db_cleanup(.data)
 
   # Apply filter to current records
+  if (!is.null(filters) && !identical(dbplyr::remote_con(filters), conn)) {
+    filters <- dplyr::copy_to(conn, filters, name = unique_table_name())
+    defer_db_cleanup(filters)
+  }
   db_table <- filter_keys(db_table, filters)
 
   # Determine the next timestamp in the data (can be NA if none is found)
@@ -155,6 +200,7 @@ update_snapshot <- function(.data, conn, db_table, timestamp, filters = NULL, me
                                 dplyr::select(.data, "checksum")) |>
       dplyr::compute() # Something has changed in dbplyr (2.2.1) that makes this compute needed.
     # Code that takes 20 secs with can be more than 30 minutes to compute without...
+    defer_db_cleanup(to_remove)
 
     nrow_to_remove <- nrow(to_remove)
 
@@ -180,7 +226,7 @@ update_snapshot <- function(.data, conn, db_table, timestamp, filters = NULL, me
 
 
   if (nrow_to_remove > 0) {
-    dplyr::rows_update(x = dplyr::tbl(conn, db_table_id, check_from = FALSE),
+    dplyr::rows_update(x = dplyr::tbl(conn, db_table_id),
                        y = to_remove,
                        by = c("checksum", "from_ts"),
                        in_place = TRUE,
@@ -192,9 +238,7 @@ update_snapshot <- function(.data, conn, db_table, timestamp, filters = NULL, me
   logger$log_info("Adding new records")
 
   if (nrow_to_add > 0) {
-    dplyr::rows_append(x = dplyr::tbl(conn, db_table_id, check_from = FALSE),
-                       y = to_add,
-                       in_place = TRUE)
+    dplyr::rows_append(x = dplyr::tbl(conn, db_table_id), y = to_add, in_place = TRUE)
   }
 
   logger$log_to_db(n_insertions = nrow_to_add)
@@ -203,13 +247,13 @@ update_snapshot <- function(.data, conn, db_table, timestamp, filters = NULL, me
 
   # If several updates come in a single day, some records may have from_ts = until_ts.
   # We remove these records here
-  redundant_rows <- dplyr::tbl(conn, db_table_id, check_from = FALSE) |>
+  redundant_rows <- dplyr::tbl(conn, db_table_id) |>
     dplyr::filter(.data$from_ts == .data$until_ts) |>
     dplyr::select("checksum", "from_ts")
   nrow_redundant <- nrow(redundant_rows)
 
   if (nrow_redundant > 0) {
-    dplyr::rows_delete(dplyr::tbl(conn, db_table_id, check_from = FALSE),
+    dplyr::rows_delete(dplyr::tbl(conn, db_table_id),
                        redundant_rows,
                        by = c("checksum", "from_ts"),
                        in_place = TRUE, unmatched = "ignore")
@@ -220,7 +264,7 @@ update_snapshot <- function(.data, conn, db_table, timestamp, filters = NULL, me
   # checksum is the same, and from_ts / until_ts are continuous
   # We collapse these records here
   if (!enforce_chronological_order) {
-    redundant_rows <- dplyr::tbl(conn, db_table_id, check_from = FALSE) |>
+    redundant_rows <- dplyr::tbl(conn, db_table_id) |>
       filter_keys(filters)
 
     redundant_rows <- dplyr::inner_join(
@@ -234,13 +278,15 @@ update_snapshot <- function(.data, conn, db_table, timestamp, filters = NULL, me
     redundant_rows_to_delete <- redundant_rows |>
       dplyr::transmute(.data$checksum, from_ts = .data$from_ts.p) |>
       dplyr::compute()
+    defer_db_cleanup(redundant_rows_to_delete)
 
     redundant_rows_to_update <- redundant_rows |>
       dplyr::transmute(.data$checksum, from_ts = .data$from_ts, until_ts = .data$until_ts.p) |>
       dplyr::compute()
+    defer_db_cleanup(redundant_rows_to_update)
 
     if (nrow(redundant_rows_to_delete) > 0) {
-      dplyr::rows_delete(x = dplyr::tbl(conn, db_table_id, check_from = FALSE),
+      dplyr::rows_delete(x = dplyr::tbl(conn, db_table_id),
                          y = redundant_rows_to_delete,
                          by = c("checksum", "from_ts"),
                          in_place = TRUE,
@@ -248,7 +294,7 @@ update_snapshot <- function(.data, conn, db_table, timestamp, filters = NULL, me
     }
 
     if (nrow(redundant_rows_to_update) > 0) {
-      dplyr::rows_update(x = dplyr::tbl(conn, db_table_id, check_from = FALSE),
+      dplyr::rows_update(x = dplyr::tbl(conn, db_table_id),
                          y = redundant_rows_to_update,
                          by = c("checksum", "from_ts"),
                          in_place = TRUE,
@@ -259,9 +305,11 @@ update_snapshot <- function(.data, conn, db_table, timestamp, filters = NULL, me
   }
 
   toc <- Sys.time()
-  logger$log_to_db(end_time = !!db_timestamp(toc, conn),
-                   duration = !!format(round(difftime(toc, tic), digits = 2)), success = TRUE)
-  logger$log_info("Finished processing for table", db_table_name, tic = toc)
+  logger$finalize_db_entry()
+  logger$log_info("Finished processing for table", as.character(db_table_id), tic = toc)
 
-  return()
+  # Release table lock
+  unlock_table(conn, db_table_id, get_schema(db_table_id))
+
+  return(NULL)
 }
